package eu.jonahbauer.chat.bot.api;

import eu.jonahbauer.chat.bot.config.BotConfig;
import eu.jonahbauer.chat.bot.impl.ChatBotFactory;
import lombok.Getter;
import lombok.experimental.Accessors;
import org.jetbrains.annotations.ApiStatus;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.Objects;

@Getter
@Accessors(makeFinal = true)
public abstract class ChatBot {
    private static final ScopedValue<String> CHANNEL = ScopedValue.newInstance();
    private static final ScopedValue<Chat> CHAT = ScopedValue.newInstance();

    protected final @NotNull BotConfig config;

    protected ChatBot(@NotNull String name) {
        this(BotConfig.builder().name(name).build());
    }

    protected ChatBot(@NotNull BotConfig defaultConfig) {
        var config = ChatBotFactory.BOT_CONFIG
                .orElseThrow(() -> new IllegalCallerException("ChatBot may only be instantiated via ChatBotFactory."));
        this.config = defaultConfig.merge(config);
    }

    public final void onMessage(@NotNull Chat chat, @NotNull Message.Post message) {
        ScopedValue
                .where(CHANNEL, message.channel())
                .where(CHAT, chat)
                .run(() -> onMessage(message));
    }

    @ApiStatus.OverrideOnly
    protected abstract void onMessage(@NotNull Message.Post message);

    /**
     * Called during the normal shutdown process for a bot.
     */
    @ApiStatus.OverrideOnly
    public void onStop() {}

    protected void post(@NotNull String message) {
        post(message, null, null, null);
    }

    protected void post(@NotNull String message, @Nullable String name, @Nullable Boolean bottag, @Nullable Boolean publicId) {
        Objects.requireNonNull(message, "message");
        name = Objects.requireNonNullElse(name, this.config.getName());
        bottag = Objects.requireNonNullElse(bottag, this.config.isBotTag());
        publicId = Objects.requireNonNullElse(publicId, this.config.isPublicId());

        var chat = CHAT.orElseThrow(() -> new IllegalStateException("post() may only be called from inside onMessage(Post)"));
        var channel = CHANNEL.orElseThrow(() -> new IllegalStateException("post() may only be called from inside onMessage(Post)"));
        chat.send(channel, name, message, bottag, publicId);
    }
}
