package eu.jonahbauer.chat.bot.api;

import eu.jonahbauer.chat.bot.config.BotConfig;
import eu.jonahbauer.chat.bot.impl.ChatBotFactory;
import eu.jonahbauer.chat.bot.impl.ChatBotFactoryAccess;
import eu.jonahbauer.chat.bot.test.MockChat;
import eu.jonahbauer.chat.bot.test.TestChatBot;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Test;

import java.time.LocalDateTime;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertThrows;

public class ChatBotTest {
    private static ChatBotFactory<TestChatBot> factory;

    @BeforeAll
    static void init() {
        factory = ChatBotFactoryAccess.newInstance(TestChatBot.class, TestChatBot::new);
    }

    @Test
    void postUsesDefaultValuesFromBotConfig() {
        var chat = new MockChat();
        var post = new Message.Post(0, "Test", "Hello World!", "", LocalDateTime.MIN, 0L, null, null, "ffffff", 0);
        var post2 = new Message.Post(0, "Test", "Hello World!", "test", LocalDateTime.MIN, 0L, null, null, "ffffff", 0);

        var bot = factory.create(BotConfig.EMPTY);
        bot.setMessage("Hello, TestChatBot#1");
        bot.onMessage(chat, post);
        bot.onMessage(chat, post2);

        var bot2 = factory.create(BotConfig.builder().name("TestChatBot#2").noBotTag().noPublicId().build());
        bot2.setMessage("Hello, TestChatBot#2");
        bot2.onMessage(chat, post);
        bot2.onMessage(chat, post2);

        var messages = chat.getMessages();
        assertEquals(4, messages.size());
        assertEquals(new MockChat.Message("", "TestBot", "Hello, TestChatBot#1", true, true), messages.get(0));
        assertEquals(new MockChat.Message("test", "TestBot", "Hello, TestChatBot#1", true, true), messages.get(1));
        assertEquals(new MockChat.Message("", "TestChatBot#2", "Hello, TestChatBot#2", false, false), messages.get(2));
        assertEquals(new MockChat.Message("test", "TestChatBot#2", "Hello, TestChatBot#2", false, false), messages.get(3));
    }

    @Test
    void postMayNotBeInvokedOutsideOfOnMessage() {
        var bot = factory.create(BotConfig.EMPTY);
        assertThrows(IllegalStateException.class, () -> bot.post("Hello, World!"));
    }
}
