package eu.jonahbauer.chat.bot.impl;

import eu.jonahbauer.chat.bot.test.TestChatBot;
import eu.jonahbauer.chat.bot.config.BotConfig;
import org.junit.jupiter.api.Test;

import static org.junit.jupiter.api.Assertions.*;

public class ChatBotFactoryTest {

    @Test
    void botsCanOnlyBeInstantiatedUsingChatBotFactory() {
        assertThrows(IllegalCallerException.class, TestChatBot::new);
        assertNotNull(new ChatBotFactory<>(TestChatBot.class, TestChatBot::new).create(BotConfig.EMPTY));
    }

    @Test
    void suppliedConfigOverridesDefaultConfig() {
        var factory = new ChatBotFactory<>(TestChatBot.class, TestChatBot::new);

        var bot = factory.create(BotConfig.EMPTY);
        assertEquals(TestChatBot.DEFAULT_CONFIG, bot.getConfig());

        var overridden = factory.create(BotConfig.builder().name("Overridden").value(TestChatBot.TEST, "overridden").build());
        assertEquals("Overridden", overridden.getConfig().require(BotConfig.NAME));
        assertEquals("overridden", overridden.getConfig().require(TestChatBot.TEST));
    }

}
