package eu.jonahbauer.chat.bot.database.impl;

import eu.jonahbauer.chat.bot.database.Entity;
import org.jetbrains.annotations.NotNull;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.Collections;

final class InserterFactory extends AbstractFactory<@NotNull Inserter<?>> {
    public static final @NotNull InserterFactory INSTANCE = new InserterFactory();

    private InserterFactory() {}

    @Override
    protected <T extends Record> @NotNull Inserter<?> computeValue0(@NotNull Class<T> type) {
        var isEntity = Entity.class.isAssignableFrom(type);
        var table = Helper.getQuotedTableName(type);
        var preparer = Preparer.get(type);
        var columns = preparer.columns();

        var query = STR."""
                    INSERT INTO \{table} (\{String.join(", ", columns)})
                    VALUES (\{String.join(", ", Collections.nCopies(columns.size(), "?"))})
                    \{isEntity ? "RETURNING id" : "RETURNING -1"}
                    """;

        return (Connection connection, T entity) -> {
            Helper.logSqlStatement(query);
            try (var stmt = connection.prepareStatement(query)) {
                preparer.prepare(stmt, entity);
                try (var result = stmt.executeQuery()) {
                    if (!result.next()) throw new SQLException();
                    return result.getLong(1);
                }
            }
        };
    }
}
