package eu.jonahbauer.chat.bot.pizza.model.view;

import eu.jonahbauer.chat.bot.pizza.PizzaBotException;
import eu.jonahbauer.chat.bot.pizza.model.table.Order;
import eu.jonahbauer.chat.bot.pizza.model.table.User;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.time.Instant;

public record OrderDetail(
        // Order
        long id,
        @NotNull String name,
        @NotNull Instant deadline,
        @Nullable Double payment,
        // Event
        long eventId,
        @NotNull String eventName,
        long adminId,
        // Restaurant
        long restaurantId,
        @NotNull String restaurantName
) {
    public @NotNull Order order() {
        return new Order(id, eventId, restaurantId, name, deadline, payment);
    }

    public @NotNull OrderDetail checkDeadline() {
        if (Instant.now().isAfter(deadline())) {
            throw new PizzaBotException("Du kannst diese Bestellung nicht mehr bearbeiten, da der Bestellannahmeschluss schon vorbei ist.");
        }
        return this;
    }

    public @NotNull OrderDetail checkAdminAccess(@NotNull User user) {
        if (adminId() != user.id()) {
            throw new PizzaBotException("Du bist nicht berechtigt, diese Aktion durchzuführen.");
        }
        return this;
    }
}
