package eu.jonahbauer.chat.server.management;

import eu.jonahbauer.chat.server.management.annotations.ManagedAttribute;
import eu.jonahbauer.chat.server.management.annotations.ManagedOperation;
import eu.jonahbauer.chat.server.management.annotations.ManagedOperation.Impact;
import eu.jonahbauer.chat.server.management.annotations.ManagedParameter;
import lombok.SneakyThrows;
import org.jetbrains.annotations.NotNull;

import javax.management.MXBean;
import javax.management.ObjectName;
import java.util.SortedSet;

@SuppressWarnings("unused")
@MXBean
public interface SocketManagerMXBean {
    @NotNull ObjectName NAME = getObjectName();

    @SneakyThrows
    private static @NotNull ObjectName getObjectName() {
        return ObjectName.getInstance("eu.jonahbauer.chat.bot.server", "component", "Sockets");
    }

    @ManagedOperation(description = "updates the credentials used to authenticate to the chat", impact = Impact.ACTION)
    void setCredentials(
            @ManagedParameter(name = "username") @NotNull String username,
            @ManagedParameter(name = "password") @NotNull String password
    );

    @ManagedOperation(description = "starts a socket for the given channel", impact = Impact.ACTION)
    void start(@ManagedParameter(name = "channel") @NotNull String channel);

    @ManagedOperation(description = "stops the socket for the given channel and waits for it to finish", impact = Impact.ACTION)
    void stop(@ManagedParameter(name = "channel") @NotNull String channel) throws InterruptedException;

    @ManagedOperation(description = "stops all sockets and waits for them to finish", impact = Impact.ACTION)
    void stop() throws InterruptedException;

    @ManagedAttribute(description = "the list of currently connected channels")
    @NotNull SortedSet<@NotNull String> getChannels();
}
