package eu.jonahbauer.chat.server.management.impl;

import eu.jonahbauer.chat.bot.config.BotConfig;
import eu.jonahbauer.chat.server.bot.ChatBotSupervisor;
import eu.jonahbauer.chat.server.management.BotConfigSupport;
import eu.jonahbauer.chat.server.management.ChatBotSupportMXBean;
import lombok.Getter;
import lombok.extern.slf4j.Slf4j;
import org.jetbrains.annotations.NotNull;

import java.lang.management.ManagementFactory;
import java.util.Objects;

@Getter
@Slf4j
public final class ChatBotSupport implements ChatBotSupportMXBean {
    private final @NotNull ChatBotSupervisor supervisor;
    private final @NotNull String name;
    private final @NotNull BotConfigSupport config;

    public static void register(@NotNull ChatBotSupervisor supervisor, @NotNull String name, @NotNull BotConfig config) {
        try {
            var impl = new ChatBotSupport(supervisor, name, config);
            var server = ManagementFactory.getPlatformMBeanServer();
            server.registerMBean(new AdvancedMBean(impl), ChatBotSupportMXBean.getObjectName(name));
        } catch (Exception ex) {
            log.error("Could not register bot as an MBean.", ex);
        }
    }

    public static void unregister(@NotNull String name) {
        try {
            var server = ManagementFactory.getPlatformMBeanServer();
            server.unregisterMBean(ChatBotSupportMXBean.getObjectName(name));
        } catch (Exception ex) {
            log.error("Could not unregister bot as an MBean.", ex);
        }
    }

    private ChatBotSupport(@NotNull ChatBotSupervisor supervisor, @NotNull String name, @NotNull BotConfig config) {
        this.supervisor = Objects.requireNonNull(supervisor);
        this.name = Objects.requireNonNull(name);
        this.config = new BotConfigSupport(config);
    }

    public void stop() throws InterruptedException {
        supervisor.stop(name);
    }
}
